const { declare } = require('@babel/helper-plugin-utils');
const presetEnv = require('@babel/preset-env');
const presetReact = require('@babel/preset-react');
const presetTypeScript = require('@babel/preset-typescript');
const decorators = require('@babel/plugin-proposal-decorators');
const classProperties = require('@babel/plugin-proposal-class-properties');
const logicalAssignmentOperators = require('@babel/plugin-proposal-logical-assignment-operators');
const nullishCoalescingOperator = require('@babel/plugin-proposal-nullish-coalescing-operator');
const numericSeparator = require('@babel/plugin-proposal-numeric-separator');
const optionalChaining = require('@babel/plugin-proposal-optional-chaining');
const privateMethods = require('@babel/plugin-proposal-private-methods');
const transformRuntime = require('@babel/plugin-transform-runtime');
const reactConstantElements = require('@babel/plugin-transform-react-constant-elements');
const reactInlineElements = require('@babel/plugin-transform-react-inline-elements');

module.exports = declare((api, options) => {
  // Cache in development mode
  const isDev = api.env('development');
  api.cache.using(() => isDev);

  // Get user options with default
  const { debug = false, react = false, typescript = false } = options;

  // Initial babel presets with preset-env
  const presets = [
    [
      presetEnv,
      {
        debug,
        loose: true,
        modules: false,
        useBuiltIns: 'usage',
        corejs: 3,
      },
    ],
  ];

  // Initial babel plugins
  const plugins = [
    [decorators, { decoratorsBeforeExport: true }],
    [classProperties, { loose: true }],
    [logicalAssignmentOperators, {}],
    [nullishCoalescingOperator, { loose: true }],
    [numericSeparator, {}],
    [optionalChaining, { loose: true }],
    [privateMethods, { loose: true }],
    [transformRuntime, {}],
  ];

  // Support react
  if (react) {
    presets.push([presetReact, { development: isDev }]);
  }

  // Support typescript
  if (typescript) {
    presets.push([
      presetTypeScript,
      {
        isTSX: react,
        allExtensions: react,
        allowNamespaces: true,
      },
    ]);
  }

  // Optimize react in production
  if (react && !isDev) {
    plugins.push([reactConstantElements, {}]);
    plugins.push([reactInlineElements, {}]);
  }

  return {
    presets,
    plugins,
    sourceMaps: isDev ? 'inline' : true,
  };
});
